#!/bin/ksh

set -u

function checkversion
{
    if [[ -f ${TOPHPU}/bin/db2hpu ]] ; then
        VERSION=`${TOPHPU}/bin/db2hpu -v 2>&1`
        VERSION2=${VERSION##* }
        if [[ "${VERSION2}" > "${HOTVERSION}" ]] ; then
            echo "Version ${VERSION2} is too high to apply the Interim Fix ${HOTFIX}"
            return 8
        else
            if [[ "${VERSION2}" = "${HOTVERSION}" ]] ; then
                echo "Version ${VERSION2} is already installed"
                return 8
            else
                if [[ "${VERSION2}" < "${HOTBASE}" ]] ; then 
                   echo "Version ${VERSION2} is too low to apply the Interim Fix ${HOTFIX}"
                   return 8
                else
                   echo "Found Version ${VERSION2}, Interim Fix ${HOTFIX} can be applied."
                   return 0
                fi
            fi
        fi
    else
        echo "Error: Could not verify the version"
        return 8
    fi
}    


function updateplugin
{
    ZIP=${TOPHPU}/install/zip

    DB2VER=`${GREP} db2version ${TOPHPU}/cfg/db2hpu.cfg | ${AWK} -F'=' '{print $2}'`
    if [[ ${DB2VER} = "" ]] ; then
        echo "Error: No plugin needed"
        return 8
    fi

    SQLDIR=`${GREP} insthomes ${TOPHPU}/cfg/db2hpu.cfg | ${AWK} -F':' '{print $2}' | ${AWK} -F',' '{print $1}'`/sqllib
    case ${DB2VER} in
    "V7" )
        DB2DIR=${SQLDIR}/cc
        ;;
    "V8" )
        DB2DIR=${SQLDIR}/tools
        ;;
    * )
        echo "Error: Unknown DB2 plugin version ${DB2VER} in configuration file"
        return 8
        ;;
    esac
    if [[ ! -a ${DB2DIR} ]] ; then
        echo "Error: Plugin install directory ${DB2DIR} not found"
        return 8
    fi
    if [[ ! -f ${DB2DIR}/db2plug.zip ]] ; then
        echo "Error: Plugin not found"
        return 8
    fi

    if [[ ! -x ${DIRBASE}/java/${DB2VER}/classes/com ]] ; then
        echo "Error: Plugin updated classes not found"
        return 8
    fi

    if [[ -a ${TOPHPU}/java/${DB2VER}/classes_${HOTFIX} ]] ; then
        echo "Error: Plugin already updated with Interim Fix ${HOTFIX}"
        return 8
    fi

    DIRCUR=`pwd`
    cd ${TOPHPU}/java/${DB2VER}
    cp -pr classes classes_${HOTFIX}
    cp -pr ${DIRBASE}/java/${DB2VER}/classes/com classes
    cd classes
    ${ZIP} -rq ${DB2DIR}/db2plug.zip -d com/ibm/inz*
    ${ZIP} -rq0 ${DB2DIR}/db2plug.zip . -i *.class
    cd ${DIRCUR}

    return 0
}


function updatepluginresources
{
    DB2VER=`${GREP} db2version ${TOPHPU}/cfg/db2hpu.cfg | ${AWK} -F'=' '{print $ 2}'`
    if [[ ${DB2VER} = "" ]] ; then
        echo "Error: No plugin ${RESOURCENAME} needed"
        return 8
    fi

    DIRCUR=`pwd`
    cd ${TOPHPU}/${RESOURCEDIR}
    cp -p ${DB2VER}/* .
    cd ${DIRCUR}

    return 0
}


AWK=/bin/awk
GREP=/bin/grep
DIRBASE=`dirname $0`
if [[ `echo ${DIRBASE} | cut -c 1` = "." ]] ; then
    DIRBASE=`pwd``echo ${DIRBASE} | cut -c 2-`
else
    if [[ `echo ${DIRBASE} | cut -c 1` != "/" ]] ; then
        DIRBASE=`pwd`/${DIRBASE}
    fi
fi

# Get install parameters
. ${DIRBASE}/parmlist.txt

# Get the OS
OS=`uname -s`

echo "------ High Performance Unload for DB2"
echo "-"
echo "------ Interim Fix ${HOTFIX} for ${HOTOS}"
echo "-"
if [[ "${OS}" != "${HOTOS}" ]] ; then
    echo "Error: This Fix is for ${HOTOS}, and this is ${OS}"
    exit 8
fi
echo "- The Interim Fix will be applied to the current version,"
echo "- located at: ${TOPHPU}"
echo "-"
echo "- The following actions will be taken"
while read FILENAME ; do
    case ${FILENAME} in
    "daemonstop" )
        echo "Stop the daemon"
        ;;
    "daemonstart" )
        echo "Start the daemon"
        ;;
    "plugin" )
        echo "Update the plugin"
        ;;
    "cfg" )
        echo "Update the config file"
        ;;
    "pluginbin" )
        echo "Update plugin binaries"
        ;;
    "pluginhelp" )
        echo "Update plugin help"
        ;;
    "pluginimages" )
        echo "Update plugin images"
        ;;
    "pluginmsg" )
        echo "Update plugin messages"
        ;;
    * )
        if [[ -f ${TOPHPU}/${FILENAME} ]] ; then
            echo "Update ${TOPHPU}/${FILENAME}"
        else
            echo "Create ${TOPHPU}/${FILENAME}"
        fi
        ;;
    esac
done < ${DIRBASE}/filelist.txt
echo "-"
echo "- All replaced files will be renamed by adding the extention _${HOTFIX}"
echo "- "
echo "- YOU NEED TO BE ROOT to run this Interim Fix"
echo "-"
echo "- Type 'install' to install the Interim Fix or 'quit' to abort"
ANSWER=" "
read ANSWER
case ${ANSWER} in
"install" )
    echo "- Installing Interim Fix ${HOTFIX}...."
    if  [[ ! -d ${TOPHPU} ]]  ; then
        echo "Fatal Error: ${TOPHPU} does not exist or is not a directory"
        exit 8
    fi
    checkversion
    if [[ $? -ne 0 ]] ; then
        exit 8
    fi
    if [[ -a ${TOPHPU}/ptf ]] ; then
        if [[ -d ${TOPHPU}/ptf ]] ; then
            if [[ -a ${TOPHPU}/ptf/${HOTFIX} ]] ; then
                if [[ -d ${TOPHPU}/ptf/${HOTFIX} ]] ; then
                    echo ""
                else
                    echo "Error: ${TOPHPU}/ptf/${HOTFIX} exists and is not a directory"
                    exit 8
                fi
            else
                mkdir ${TOPHPU}/ptf/${HOTFIX}
            fi
        else
           echo "Error: ${TOPHPU}/ptf/${HOTFIX} exists and is not a directory"
           exit 8
        fi
    else
        mkdir ${TOPHPU}/ptf
        mkdir ${TOPHPU}/ptf/${HOTFIX}
    fi
    cp ${DIRBASE}/filelist.txt ${TOPHPU}/ptf/${HOTFIX}/filelist_${HOTFIX}
    cp -p ${DIRBASE}/uninstall.sh ${TOPHPU}/ptf/${HOTFIX}/uninstall.sh
    
    while read FILENAME ; do
        case ${FILENAME} in
        "daemonstop" )
            echo "Stopping the daemon"
            ${TOPHPU}/bin/db2hpud stop
            ;;
        "daemonstart" )
            echo "Starting the daemon"
            ${TOPHPU}/bin/db2hpud start
            ;;
        "plugin" )
            echo "Updating the plugin"
            updateplugin
            ;;
        "cfg" )
            echo "Updating the config file"
            ;;
        "pluginbin" )
            echo "Updating plugin binaries"
            RESOURCEDIR=bin
            RESOURCENAME=binaries
            updatepluginresources
            ;;
        "pluginhelp" )
            echo "Updating plugin help"
            RESOURCEDIR=help
            RESOURCENAME=help
            updatepluginresources
            ;;
        "pluginimages" )
            echo "Updating plugin images"
            RESOURCEDIR=images
            RESOURCENAME=images
            updatepluginresources
            ;;
        "pluginmsg" )
            echo "Updating plugin messages"
            RESOURCEDIR=msg
            RESOURCENAME=messages
            updatepluginresources
            ;;
        * )
            if [[ -f ${TOPHPU}/${FILENAME}_${HOTFIX} ]] ; then
                echo "Error: ${TOPHPU}/${FILENAME} already updated with Interim Fix ${HOTFIX}"
            else
                if [[ -f ${TOPHPU}/${FILENAME} ]] ; then
                    mv ${TOPHPU}/${FILENAME} ${TOPHPU}/${FILENAME}_${HOTFIX}
                    echo "Updating ${TOPHPU}/${FILENAME}"     
                else
#                    echo "Warning File ${TOPHPU}/${FILENAME} not Present"
                    echo "Creating ${TOPHPU}/${FILENAME}"     
                fi
                cp -p ${DIRBASE}/${FILENAME} ${TOPHPU}/${FILENAME}
            fi
            ;;
        esac
    done < ${DIRBASE}/filelist.txt
    echo ""
    echo "Installation completed..."
    exit 0
    ;;   

"quit"|* )
    echo "- Installation cancelled.... No update done."
    exit 1
    ;;
esac
   

