#!/bin/ksh

set -u

function restoreplugin
{
    ZIP=${TOPHPU}/install/zip

    DB2VER=`${GREP} db2version ${TOPHPU}/cfg/db2hpu.cfg | ${AWK} -F'=' '{print $2}'`
    if [[ ${DB2VER} = "" ]] ; then
        echo "Error: No plugin needed"
        return 8
    fi

    SQLDIR=`${GREP} insthomes ${TOPHPU}/cfg/db2hpu.cfg | ${AWK} -F':' '{print $2}' | ${AWK} -F',' '{print $1}'`/sqllib
    case ${DB2VER} in
    "V7" )
        DB2DIR=${SQLDIR}/cc
        ;;
    "V8" )
        DB2DIR=${SQLDIR}/tools
        ;;
    * )
        echo "Error: Unknown DB2 plugin version ${DB2VER} in configuration file"
        return 8
        ;;
    esac
    if [[ ! -a ${DB2DIR} ]] ; then
        echo "Error: Plugin install directory ${DB2DIR} not found"
        return 8
    fi
    if [[ ! -f ${DB2DIR}/db2plug.zip ]] ; then
        echo "Error: Plugin not found"
        return 8
    fi

    if [[ ! -a ${TOPHPU}/java/${DB2VER}/classes_${HOTFIX} ]] ; then
        echo "Error: Plugin already restored from Interim Fix ${HOTFIX}"
        return 8
    fi

    DIRCUR=`pwd`
    cd ${TOPHPU}/java/${DB2VER}
    rm -fR classes
    cp -pr classes_${HOTFIX} classes
    rm -fR classes_${HOTFIX}
    cd classes
    ${ZIP} -rq ${DB2DIR}/db2plug.zip -d com/ibm/inz*
    ${ZIP} -rq0 ${DB2DIR}/db2plug.zip . -i *.class
    cd ${DIRCUR}

    return 0
}


function restorepluginresources
{
    DB2VER=`${GREP} db2version ${TOPHPU}/cfg/db2hpu.cfg | ${AWK} -F'=' '{print $ 2}'`
    if [[ ${DB2VER} = "" ]] ; then
        echo "Error: No plugin ${RESOURCENAME} needed"
        return 8
    fi

    DIRCUR=`pwd`
    cd ${TOPHPU}/${RESOURCEDIR}
    cp -p ${DB2VER}/* .
    cd ${DIRCUR}

    return 0
}


AWK=/bin/awk
GREP=/bin/grep
DIRBASE=`dirname $0`
if [[ ${DIRBASE} != . ]] ; then
    echo "Error: Interim Fix uninstall must be executed from its own directory "
    exit 8
fi

HOTFIX=`basename ${PWD}`
if [[ ! -f ${DIRBASE}/filelist_${HOTFIX} ]] ; then
    echo "Error: rollback filelist is not available"
    exit 8
fi

echo "------ High Performance Unload for DB2"
echo "-"
echo "- The Interim Fix ${HOTFIX} will be uninstalled"
echo "- from: ${TOPHPU}"
echo "-"
echo "- The following actions will be taken"
while read FILENAME ; do
    case ${FILENAME} in
    "daemonstop" )
        echo "Stop the daemon"
        ;;
    "daemonstart" )
        echo "Start the daemon"
        ;;
    "plugin" )
        echo "Restore the plugin"
        ;;
    "cfg" )
        echo "Restore the config file"
        ;;
    "pluginbin" )
        echo "Restore plugin binaries"
        ;;
    "pluginhelp" )
        echo "Restore plugin help"
        ;;
    "pluginimages" )
        echo "Restore plugin images"
        ;;
    "pluginmsg" )
        echo "Restore plugin messages"
        ;;
    * )
        if [[ -f ${TOPHPU}/${FILENAME}_${HOTFIX} ]] ; then
            echo "Restore ${TOPHPU}/${FILENAME}"
        else
            echo "Delete ${TOPHPU}/${FILENAME}"
        fi
        ;;
    esac
done < ${DIRBASE}/filelist_${HOTFIX}


echo "- "
echo "- YOU NEED TO BE ROOT to run this Interim Fix"
echo "-"
echo "- Type 'uninstall' to uninstall the Interim Fix or 'quit' to abort"
ANSWER=" "
read ANSWER
case ${ANSWER} in
"uninstall" )
    echo "- Uninstalling Interim Fix ${HOTFIX}...."
    while read FILENAME ; do
        case ${FILENAME} in
        "daemonstop" )
            echo "Stopping the daemon"
            ${TOPHPU}/bin/db2hpud stop
            ;;
        "daemonstart" )
            echo "Starting the daemon"
            ${TOPHPU}/bin/db2hpud start
            ;;
        "plugin" )
            echo "Restoring the plugin"
            restoreplugin
            ;;
        "cfg" )
            echo "Restoring the config file"
            ;;
        "pluginbin" )
            echo "Restoring plugin binaries"
            RESOURCEDIR=bin
            RESOURCENAME=binaries
            restorepluginresources
            ;;
        "pluginhelp" )
            echo "Restoring plugin help"
            RESOURCEDIR=help
            RESOURCENAME=help
            restorepluginresources
            ;;
        "pluginimages" )
            echo "Restoring plugin images"
            RESOURCEDIR=images
            RESOURCENAME=images
            restorepluginresources
            ;;
        "pluginmsg" )
            echo "Restoring plugin messages"
            RESOURCEDIR=msg
            RESOURCENAME=messages
            restorepluginresources
            ;;
        * )
            if [[ -f ${TOPHPU}/${FILENAME}_${HOTFIX} ]] ; then
                echo "Restoring ${TOPHPU}/${FILENAME}"     
                rm ${TOPHPU}/${FILENAME}
                mv ${TOPHPU}/${FILENAME}_${HOTFIX} ${TOPHPU}/${FILENAME}
            else
                echo "Deleting ${TOPHPU}/${FILENAME}"     
                rm ${TOPHPU}/${FILENAME}
            fi
            ;;
        esac
    done < ${DIRBASE}/filelist_${HOTFIX}
    echo ""
    echo "Uninstallation completed..."
    exit 0
    ;;

"quit"|* )
    echo "- Uninstallation cancelled.... No update done."
    exit 1
    ;;
esac


